/*
 * Copyright (c) 2016 Intel Corporation.
 *
 * SPDX-License-Identifier: Apache-2.0
 */

/* Sample to illustrate the usage of crypto APIs. The sample plaintext
 * and ciphertexts used for crosschecking are from TinyCrypt.
 */

#include <device.h>
#include <zephyr.h>
#include <string.h>
#include "mbedtls/ecp.h"
#include "mbedtls/ecdh.h"
#include <syscalls/rand32.h>
#include "mbedtls/error.h"
#include "mbedtls/entropy.h"
#include "mbedtls/ctr_drbg.h"


#define LOG_LEVEL CONFIG_CRYPTO_LOG_LEVEL
#include <logging/log.h>
LOG_MODULE_REGISTER(main);

int random_gen(void *p_entropy, unsigned char *dest, size_t size) {
    printk("In random!\n");
	int r = 0;
	r = sys_csrand_get(dest, size); //crashes here and reboots
	if (!r){
		printk("success!\n");
	}

	return 1;
}



void main(void)
{
	mbedtls_ecdh_context ctx_cli, ctx_srv;
	unsigned char cli_to_srv[32], srv_to_cli[32];
	mbedtls_entropy_context entropy;
    mbedtls_ctr_drbg_context ctr_drbg;
	
	mbedtls_ecdh_init( &ctx_cli );
    mbedtls_ecdh_init( &ctx_srv );
    mbedtls_ctr_drbg_init( &ctr_drbg );
	mbedtls_entropy_init( &entropy );
	const char pers[] = "ecdh";	

    signed int ret;
    
	// if( ( ret = mbedtls_ctr_drbg_seed( &ctr_drbg, mbedtls_entropy_func, &entropy,
    //                            (const unsigned char *) pers,
    //                            sizeof pers ) ) != 0 )
	if( ( ret = mbedtls_ctr_drbg_seed( &ctr_drbg, random_gen, &entropy,
                               (const unsigned char *) pers,
                               sizeof pers ) ) != 0 )
    {
        printk( " failed\n  ! mbedtls_ctr_drbg_seed returned %x\n", ret);
        // goto exit;
		char buffer[100];
		// mbedtls_strerror( ret, buffer, sizeof(buffer) );
		// printk("Error string = %s\n", buffer);
    }


	mbedtls_ecdh_setup( &ctx_cli, MBEDTLS_ECP_DP_SECP521R1 );
	mbedtls_ecp_group_init( &ctx_cli.grp );

    // ret = mbedtls_ecdh_gen_public( &ctx_cli.grp, &ctx_cli.d, &ctx_cli.Q,
    //                                mbedtls_entropy_func, NULL );
	ret = mbedtls_ecdh_gen_public( &ctx_cli.grp, &ctx_cli.d, &ctx_cli.Q,
                                   random_gen, NULL );							   

	printk("Returned: %x\n", ret);

}
